@@ -1172,6 +1172,7 @@ class Cover(ImageView):  # {{{
                 pm = QPixmap(I('default_cover.png'))
             else:
                 self._cdata = cdata
+            pm.setDevicePixelRatio(getattr(self, 'devicePixelRatioF', self.devicePixelRatio)())
             self.setPixmap(pm)
             tt = _('This book has no cover')
             if self._cdata:
@@ -1286,6 +1287,7 @@ class TagsEdit(EditWithComplete, ToMetadataMixin):  # {{{
 
     def __init__(self, parent):
         EditWithComplete.__init__(self, parent)
+        self.lineEdit().setMaxLength(65536)  # see https://bugs.launchpad.net/bugs/1569388
         self.books_to_refresh = set([])
         self.setToolTip(self.TOOLTIP)
         self.setWhatsThis(self.TOOLTIP)
@@ -1378,6 +1380,7 @@ class LanguagesEdit(LE, ToMetadataMixin):  # {{{
         cv = self.current_val
         if cv != self.original_val:
             db.set_languages(id_, cv)
+        self.update_recently_used()
 # }}}
 
 # Identifiers {{{
@@ -1522,9 +1525,11 @@ class IdentifiersEdit(QLineEdit, ToMetadataMixin):
             text = d.text()
             if not text:
                 return
-        vals = self.current_val
-        vals['isbn'] = text
-        self.current_val = vals
+        text = check_isbn(text)
+        if text:
+            vals = self.current_val
+            vals['isbn'] = text
+            self.current_val = vals
 
 # }}}
 
