@@ -117,7 +117,7 @@ class WhereBox(QComboBox):
 
     def __init__(self, parent, emphasize=False):
         QComboBox.__init__(self)
-        self.addItems([_('Current file'), _('All text files'), _('All style files'), _('Selected files'), _('Marked text')])
+        self.addItems([_('Current file'), _('All text files'), _('All style files'), _('Selected files'), _('Open files'), _('Marked text')])
         self.setToolTip('<style>dd {margin-bottom: 1.5ex}</style>' + _(
             '''
             Where to search/replace:
@@ -130,6 +130,8 @@ class WhereBox(QComboBox):
             <dd>Search in all style (CSS) files</dd>
             <dt><b>Selected files</b></dt>
             <dd>Search in the files currently selected in the Files Browser</dd>
+            <dt><b>Open files</b></dt>
+            <dd>Search in the files currently open in the editor</dd>
             <dt><b>Marked text</b></dt>
             <dd>Search only within the marked text in the currently opened file. You can mark text using the Search menu.</dd>
             </dl>'''))
@@ -142,7 +144,7 @@ class WhereBox(QComboBox):
 
     @dynamic_property
     def where(self):
-        wm = {0:'current', 1:'text', 2:'styles', 3:'selected', 4:'selected-text'}
+        wm = {0:'current', 1:'text', 2:'styles', 3:'selected', 4:'open', 5:'selected-text'}
         def fget(self):
             return wm[self.currentIndex()]
         def fset(self, val):
@@ -1185,7 +1187,7 @@ def initialize_search_request(state, action, current_editor, current_editor_name
     marked = False
     if where == 'current':
         editor = current_editor
-    elif where in {'styles', 'text', 'selected'}:
+    elif where in {'styles', 'text', 'selected', 'open'}:
         files = searchable_names[where]
         if current_editor_name in files:
             # Start searching in the current editor
@@ -1320,16 +1322,21 @@ def run_search(
         return no_replace(_(
                 'Currently selected text does not match the search query.'))
 
-    def count_message(action, count, show_diff=False):
-        msg = _('%(action)s %(num)s occurrences of %(query)s') % dict(num=count, query=errfind, action=action)
-        if show_diff and count > 0:
-            d = MessageBox(MessageBox.INFO, _('Searching done'), prepare_string_for_xml(msg), parent=gui_parent, show_copy_button=False)
-            d.diffb = b = d.bb.addButton(_('See what &changed'), d.bb.ActionRole)
-            b.setIcon(QIcon(I('diff.png'))), d.set_details(None), b.clicked.connect(d.accept)
-            b.clicked.connect(partial(show_current_diff, allow_revert=True), type=Qt.QueuedConnection)
-            d.exec_()
-        else:
-            info_dialog(gui_parent, _('Searching done'), prepare_string_for_xml(msg), show=True)
+    def count_message(replaced, count, show_diff=False, show_dialog=True):
+        if show_dialog:
+            if replaced:
+                msg = _('Performed the replacement at {num} occurrences of {query}')
+            else:
+                msg = _('Found {num} occurrences of {query}')
+            msg = msg.format(num=count, query=errfind)
+            if show_diff and count > 0:
+                d = MessageBox(MessageBox.INFO, _('Searching done'), prepare_string_for_xml(msg), parent=gui_parent, show_copy_button=False)
+                d.diffb = b = d.bb.addButton(_('See what &changed'), d.bb.ActionRole)
+                b.setIcon(QIcon(I('diff.png'))), d.set_details(None), b.clicked.connect(d.accept)
+                b.clicked.connect(partial(show_current_diff, allow_revert=True), type=Qt.QueuedConnection)
+                d.exec_()
+            else:
+                info_dialog(gui_parent, _('Searching done'), prepare_string_for_xml(msg), show=True)
 
     def do_all(replace=True):
         count = 0
@@ -1376,7 +1383,7 @@ def run_search(
                 with current_container().open(n, 'wb') as f:
                     f.write(raw.encode('utf-8'))
         QApplication.restoreOverrideCursor()
-        count_message(_('Replaced') if replace else _('Found'), count, show_diff=replace)
+        count_message(replace, count, show_diff=replace)
         return count
 
     with BusyCursor():
@@ -1388,7 +1395,12 @@ def run_search(
             return do_find()
         if action == 'replace-all':
             if marked:
-                return count_message(_('Replaced'), sum(editor.all_in_marked(p, repl) for p, repl in searches))
+                show_result_dialog = True
+                for p, repl in searches:
+                    if getattr(getattr(repl, 'func', None), 'suppress_result_dialog', False):
+                        show_result_dialog = False
+                        break
+                return count_message(True, sum(editor.all_in_marked(p, repl) for p, repl in searches), show_dialog=show_result_dialog)
             add_savepoint(_('Before: Replace all'))
             count = do_all()
             if count == 0:
@@ -1398,7 +1410,7 @@ def run_search(
             return
         if action == 'count':
             if marked:
-                return count_message(_('Found'), sum(editor.all_in_marked(p) for p, __ in searches))
+                return count_message(False, sum(editor.all_in_marked(p) for p, __ in searches))
             return do_all(replace=False)
 
 if __name__ == '__main__':
